-- CONTROL
-- The commands for changing Clone Fighter's behaviour. Load first.

-- clone_prefstats
-- Whitelists a certain stat combo to use. If used, Clone Fighter will not switch off of that stat combo.
COM_AddCommand("clone_prefstats", function(p, speed, weight, pnum)
	local pp = pnum and tonumber(pnum)-1 or 0
	if pp > 3 or pp < 0 then pp = 0 end
	
	local player = displayplayers[pp] or p
	if pnum then
		CONS_Printf(p, "Player "..tostring(pp+1)..",")
	end
	
	if not player.cloneFighter then
		CONS_Printf(p, "You can only use this as Clone Fighter.")
		CONS_Printf(p, "If you're getting this as Clone Fighter, likely you're in an intermission. You can only use this in a level.")
		return
	end
	if leveltime < 10*TICRATE then
		CONS_Printf(p, "Due to technical reasons, this command can only be used after 10 seconds have passed since map load.")
		CONS_Printf(p, "Please wait "..tostring((10*TICRATE-leveltime)/TICRATE).." more seconds.")
		return
	end
	
	if speed == "clear"
	or (speed == "0" and weight == "0") then
		player.prefSpeed, player.prefWeight = nil, nil
		CONS_Printf(p, "Cleared your stat preference.")
		return
	end
	
	if tonumber(speed) == nil then
		CONS_Printf(p, "Your preferred Speed has to be a number.")
		return
	elseif tonumber(weight) == nil then
		CONS_Printf(p, "Your preferred Weight has to be a number.")
		return
	end
	local s, w = tonumber(speed), tonumber(weight)
	
	if s < 0 or s > 9 or w < 0 or w > 9 then
		CONS_Printf(p, "Invalid stats {"..speed..", "..weight.."}. Both numbers have to be from 1 to 9.")
		CONS_Printf(p, "If you want to, you can use 0 on either to still randomize that stat.")
		return
	end
	
	player.prefSpeed, player.prefWeight = s, w
	CONS_Printf(p, "Set your preferred stats to {"..speed..", "..weight.."}.")
end)

-- clone_validchars
-- Prohibits Clone Fighter from choosing characters that do not exist (i.e. going into 'undefined' mode). Admin command.
rawset(_G, "CLONE_ValidOnly", false)

COM_AddCommand("clone_validchars", function(player, arg)
	if netgame
	and (server.name != player.name
	or not IsPlayerAdmin(player)) then -- There must be a better method to this, too.
		CONS_Printf(player, "Only the server or a remote admin can use this command.")
		return
	end
	
	if arg == "yes" or arg == "on" or arg == "1" or not CLONE_ValidOnly then
		CLONE_ValidOnly = true
		print("The server has restricted Clone Fighter to using only valid characters.")
	elseif arg == "no" or arg == "off" or arg == "0" or CLONE_ValidOnly then
		CLONE_ValidOnly = false
		print("The server has lifted Clone Fighter's stat restrictions.")
	end
end)

-- clone_blacklist
-- Adds certain skins to the blacklist.
rawset(_G, "CLONE_Blacklist", {})

COM_AddCommand("clone_blacklist", function(player, skin, act)
	if netgame
	and (server.name != player.name
	or not IsPlayerAdmin(player)) then -- Again, there must be a better method.
		CONS_Printf(player, "Only the server or a remote admin can use this command.")
		return
	end
	
	if not skin
	or (not skins[skin] and skin != "all") then
		CONS_Printf(player, "Argument 1 must be a valid skin name (e.g. sonic, eggman, metalsonic).")
		return
	end
	
	if skin == 'all' then
		if not act then
			CONS_Printf(player, "State what you want to do with the skins... (add, remove)")
			CONS_Printf(player, "\x82WARNING:\x80 This might be dangerous.")
			return
		
		elseif act == "add" then
			for s in skins.iterate do
				if not CLONE_Blacklist[s.name] then CLONE_Blacklist[s.name] = true end
			end
			print("EVERYONE was added to Clone Fighter's blacklist. Have fun driving around with just randomized stats.")
		elseif act == "remove" then
			for s in skins.iterate do
				if CLONE_Blacklist[s.name] then CLONE_Blacklist[s.name] = nil end
			end
			print("Everyone was removed from Clone Fighter's blacklist.")
		else
			CONS_Printf(player, "State what you want to do with the skins... \x82(add, remove)")
			CONS_Printf(player, "\x82WARNING:\x80 This might be dangerous.")
			return
		end
	else
		if not act then
			CONS_Printf(player, "State what you want to do with "..skins[skin].realname.."... (add, remove)")
			return
		
		elseif act == "add" then
			if not CLONE_Blacklist[skin] then CLONE_Blacklist[skin] = true end
			print(skins[skin].realname.." was added to Clone Fighter's blacklist.")
		elseif act == "remove" then
			if CLONE_Blacklist[skin] then CLONE_Blacklist[skin] = nil end
			print(skins[skin].realname.." was removed from Clone Fighter's blacklist.")
		else
			CONS_Printf(player, "State what you want to do with "..skins[skin].realname.."... \x82(add, remove)")
			return
		end
	end
end)