-- CLONE FIGHTER
-- Stat (and/or character) randomizer, within a character.

-- SKIN TABLE
-- This is where C.F will grab the skins to switch to.
rawset(_G, "CLONE_SkinTable", {})

local function countSkins()
	CLONE_SkinTable = {}
	
	for i in skins.iterate()
		if i.name == "clonefighter" or i.name == "clonefighterjr" then continue end
		
		-- Speed
		CLONE_SkinTable[i.kartspeed] = $ or {}
		-- Weight
		CLONE_SkinTable[i.kartspeed][i.kartweight] = $ or {}
		table.insert(CLONE_SkinTable[i.kartspeed][i.kartweight], i.name)
	end
	-- Debug prints
	/*for speed, i in pairs(CLONE_SkinTable) do
		for weight, j in pairs(i) do
			for _, skin in pairs(j) do
				print(skin, speed..", "..weight)
			end
		end
	end*/
end

countSkins()
addHook("MapLoad", countSkins)

-- Item roll sound table, for reference
local rollTable = {sfx_itrol1, sfx_itrol2, sfx_itrol3, sfx_itrol4, sfx_itrol5, sfx_itrol6, sfx_itrol7, sfx_itrol8}

-- RANDOMIZE!
-- Clone Fighter's behaviour starts here.
addHook("MapChange", do for player in players.iterate do
	player.cfSkin, player.cfSpeed, player.cfWeight = nil, nil, nil -- Reset. Otherwise it kinda breaks.
end; end)

local function CLONE_Reroll(d, player, prefs, prefw)
	if d == 0 then
		print("\x83NOTICE:\x80 Invalid stats rolled multiple times. Undefined character.")
		S_StartSound(nil, sfx_itrolm, player)
		player.cfSkin = "undefined"
		player.kartspeed, player.kartweight = prefs, prefw
		if not prefs then player.cfSpeed = P_RandomRange(1, 9) end
		if not prefw then player.cfWeight = P_RandomRange(1, 9) end
		return
	end
	
	player.cfSpeed, player.cfWeight = prefs, prefw
	if not prefs then player.cfSpeed = P_RandomRange(1, 9) end
	if not prefw then player.cfWeight = P_RandomRange(1, 9) end
	if not CLONE_SkinTable[player.cfSpeed] or not CLONE_SkinTable[player.cfSpeed][player.cfWeight] then
		CLONE_Reroll(d-1, player, prefs, prefw)
		return
	else
		local spwgclass = CLONE_SkinTable[player.cfSpeed][player.cfWeight]
		local roll = P_RandomRange(1, #spwgclass)
		if CLONE_Blacklist[spwgclass[roll]] then
			CLONE_Reroll(d-1, player, prefs, prefw)
		else
			S_StartSound(nil, sfx_itrolf, player)
			player.mo.skin = spwgclass[roll]
			player.cfSkin = player.mo.skin
			player.kartspeed, player.kartweight = prefs, prefw
			return
		end
	end
end

addHook("ThinkFrame", do for player in players.iterate do -- Apparently no PlayerThink in Kart, so we gotta do this
	if player.mo and player.mo.skin == "clonefighter" then
		player.cloneFighter = true
	elseif (leveltime < 5 and (player.prefSpeed or player.prefWeight))
	or (leveltime < 5*TICRATE/2 and not (player.prefSpeed or player.prefWeight)) or (player.cfTimer and player.cfTimer > TICRATE/2+1) then
		player.cloneFighter = false
		player.cfTimer = 0
		player.cfChoose = ""
		player.cfSkin = nil
	end
	
	if not player.cloneFighter then continue end
	player.mo.colorized = true
	local frameFlag = FF_TRANS30
	if leveltime % 2 == 0 then frameFlag = FF_TRANS20 end
	player.mo.frame = $|FF_FULLBRIGHT|frameFlag
	
	-- Spinout effect
	if player.kartstuff[k_spinouttimer] then
		player.mo.flags2 = $|MF2_DONTDRAW
		local addx1 = player.mo.x + FixedMul(P_RandomRange(0, 15)*FRACUNIT, cos(player.mo.angle+ANGLE_90+P_RandomRange(-15,15)*ANG1))
		local addy1 = player.mo.y + FixedMul(P_RandomRange(0, 15)*FRACUNIT, sin(player.mo.angle+ANGLE_90+P_RandomRange(-15,15)*ANG1))
		local addx2 = player.mo.x + FixedMul(P_RandomRange(0, 15)*FRACUNIT, cos(player.mo.angle-ANGLE_90+P_RandomRange(-15,15)*ANG1))
		local addy2 = player.mo.y + FixedMul(P_RandomRange(0, 15)*FRACUNIT, sin(player.mo.angle-ANGLE_90+P_RandomRange(-15,15)*ANG1))
		
		for i = 1, 6
			local ghost1 = P_SpawnGhostMobj(player.mo)
			ghost1.fuse = 2
			P_TeleportMove(ghost1, addx1, addy1, player.mo.z)
			ghost1.angle = player.frameangle
			local ghost2 = P_SpawnGhostMobj(player.mo)
			ghost2.fuse = 2
			P_TeleportMove(ghost2, addx2, addy2, player.mo.z)
			ghost1.angle = player.frameangle
		end
	end
	
	if player.exiting and not player.cfTimer then
		player.cfTimer = 1
		player.cfChoose = player.mo.skin
		if player.cfChoose != "clonefighter" then S_StartSound(player.mo, sfx_s1b1) end
		player.rollIndex = 0
		player.cfSkin = nil
	end
	
	if player.cfSkin and player.cfSkin != "undefined"
	and player.mo.skin != player.cfSkin
	then player.mo.skin = player.cfSkin end
	
	if player.cfSpeed and player.cfWeight
	and (player.kartspeed != player.cfSpeed or player.kartweight != player.cfWeight)
	then player.kartspeed, player.kartweight = player.cfSpeed, player.cfWeight end
	
	if player.prefSpeed or player.prefWeight then
		-- Preference.
		if leveltime == 5 then
			player.cfSpeed, player.cfWeight = player.prefSpeed, player.prefWeight
			if player.prefSpeed == 0 then player.cfSpeed = P_RandomRange(1, 9) end
			if player.prefWeight == 0 then player.cfWeight = P_RandomRange(1, 9) end
			if not CLONE_SkinTable[player.cfSpeed] or not CLONE_SkinTable[player.cfSpeed][player.cfWeight] then
				if CLONE_ValidOnly then
					CLONE_Reroll(20, player, player.prefSpeed, player.prefWeight)
				else
					S_StartSound(nil, sfx_itrolm, player)
					player.cfSkin = "undefined"
					player.kartspeed, player.kartweight = player.cfSpeed, player.cfWeight
				end
			else
				local spwgclass = CLONE_SkinTable[player.cfSpeed][player.cfWeight]
				local roll = P_RandomRange(1, #spwgclass)
				if CLONE_Blacklist[spwgclass[roll]] then
					CLONE_Reroll(20, player, player.cfSpeed, player.cfWeight)
				else
					S_StartSound(nil, sfx_itrolf, player)
					player.mo.skin = spwgclass[roll]
					player.cfSkin = player.mo.skin
					player.kartspeed, player.kartweight = player.cfSpeed, player.cfWeight
				end
			end
		elseif leveltime <= 5+TICRATE/2 and leveltime > 5 then
			if player.cfSkin == "undefined" then
				local blinkcolor = SKINCOLOR_RED
				if player.skincolor >= SKINCOLOR_SALMON and player.skincolor <= SKINCOLOR_KETCHUP then blinkcolor = SKINCOLOR_BLUE end
				player.mo.color = (leveltime%2 + 1 == 1) and blinkcolor or player.skincolor
				if leveltime >= (5+TICRATE/2 - 1) then player.mo.color = player.skincolor end
			else
				player.mo.skin = (leveltime%2 + 1 == 1) and "clonefighter" or player.cfSkin
				if leveltime >= (5+TICRATE/2 - 1) then player.mo.skin = player.cfSkin end
			end
		end
	else
		-- No preference
		if leveltime < 5*TICRATE/2 then
			if leveltime == 1 then player.rollIndex = 1 end
			player.cfTimer = 0
			player.cfChoose = ""
			if leveltime % 3 == 0 then
				S_StartSound(nil, rollTable[player.rollIndex] or sfx_itrol1, player)
				player.rollIndex = $ and $+1 or 1
				if player.rollIndex == 9 then player.rollIndex = 1 end
				-- HUD thing. This is stupid, dumb, and there must be a better solution to this (v.random?)
				player.randomskin = P_RandomRange(1, #skins-1)
			end
		elseif leveltime == 5*TICRATE/2 then
			player.cfSpeed, player.cfWeight = P_RandomRange(1, 9), P_RandomRange(1, 9)
			if not CLONE_SkinTable[player.cfSpeed] or not CLONE_SkinTable[player.cfSpeed][player.cfWeight] then
				if CLONE_ValidOnly then
					CLONE_Reroll(20, player)
				else
					S_StartSound(nil, sfx_itrolm, player)
					player.cfSkin = "undefined"
					player.kartspeed, player.kartweight = player.cfSpeed, player.cfWeight
				end
			else
				local spwgclass = CLONE_SkinTable[player.cfSpeed][player.cfWeight]
				local roll = P_RandomRange(1, #spwgclass)
				if CLONE_Blacklist[spwgclass[roll]] then
					CLONE_Reroll(20, player, player.cfSpeed, player.cfWeight)
				else
					S_StartSound(nil, sfx_itrolf, player)
					player.mo.skin = spwgclass[roll]
					player.cfSkin = player.mo.skin
					player.kartspeed, player.kartweight = player.cfSpeed, player.cfWeight
				end
			end
		elseif leveltime <= 3*TICRATE then
			if player.cfSkin == "undefined" then
				local blinkcolor = SKINCOLOR_RED
				if player.skincolor >= SKINCOLOR_SALMON and player.skincolor <= SKINCOLOR_KETCHUP then blinkcolor = SKINCOLOR_BLUE end
				player.mo.color = (leveltime%2 + 1 == 1) and blinkcolor or player.skincolor
				if leveltime >= (3*TICRATE - 1) then player.mo.color = player.skincolor end
			else
				player.mo.skin = (leveltime%2 + 1 == 1) and "clonefighter" or player.cfSkin
				if leveltime >= (3*TICRATE - 1) then player.mo.skin = player.cfSkin end
			end
		end
	end
	
	if player.cfTimer then
		player.mo.skin = (player.cfTimer % 2 == 1) and "clonefighter" or player.cfChoose
		player.cfTimer = $+1
		if player.cfTimer >= TICRATE/2 then player.mo.skin = "clonefighter" end
	end
end; end)